<?php
/************************************************************************
 * MXChange v0.2.1                                    Start: 08/31/2008 *
 * ===============                              Last change: 08/31/2008 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : surfbar_functions.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Functions for surfbar                            *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Funktionen fuer die Surfbar                      *
 * -------------------------------------------------------------------- *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2008 by Roland Haeder                           *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	$INC = substr(dirname(__FILE__), 0, strpos(dirname(__FILE__), "/inc") + 4) . "/security.php";
	require($INC);
}

// -----------------------------------------------------------------------------
//                               Admin functions
// -----------------------------------------------------------------------------
//
// Admin has added an URL with given user id and so on
function SURFBAR_ADMIN_ADD_URL ($url, $limit) {
	// Do some pre-checks
	if (!IS_ADMIN()) {
		// Not an admin
		return false;
	} elseif (!VALIDATE_URL($url)) {
		// URL invalid
		return false;
	} elseif (SURFBAR_LOOKUP_BY_URL($url, "0")) {
		// URL already found in surfbar!
		return false;
	} elseif (!SURFBAR_IF_USER_BOOK_MORE_URLS()) {
		// No more allowed!
		return false;
	} elseif ("".($limit + 0)."" != "".$limit."") {
		// Invalid amount entered
		return false;
	}

	// Register the new URL
	return SURFBAR_REGISTER_URL($url, "0", "ACTIVE", "unlock", $limit);
}
// Admin unlocked an email so we can migrate the URL
function SURFBAR_ADMIN_MIGRATE_URL ($url, $uid) {
	// Do some pre-checks
	if (!IS_ADMIN()) {
		// Not an admin
		return false;
	} elseif (!VALIDATE_URL($url)) {
		// URL invalid
		return false;
	} elseif (SURFBAR_LOOKUP_BY_URL($url, $uid)) {
		// URL already found in surfbar!
		return false;
	} elseif (!SURFBAR_IF_USER_BOOK_MORE_URLS($uid)) {
		// No more allowed!
		return false;
	}

	// Register the new URL
	return SURFBAR_REGISTER_URL($url, $uid, "MIGRATED", "migrate");
}
// Admin function for unlocking URLs
function SURFBAR_ADMIN_UNLOCK_URL_IDS ($IDs) {
	// Is this an admin or invalid array?
	if (!IS_ADMIN()) {
		// Not admin or invalid IDs array
		return false;
	} elseif (!is_array($IDs)) {
		// No array
		return false;
	} elseif (count($IDs) == 0) {
		// Empty array
		return false;
	}

	// Set to true to make AND expression valid if first URL got unlocked
	$done = true;

	// Update the status for all ids
	foreach ($IDs as $id => $dummy) {
		// Test all ids through (ignores failed)
		$done = (($done) && (SURFBAR_CHANGE_STATUS($id, "PENDING", "ACTIVE")));
	} // END - if

	// Return total status
	return $done;
}
// Admin function for rejecting URLs
function SURFBAR_ADMIN_REJECT_URL_IDS ($IDs) {
	// Is this an admin or invalid array?
	if (!IS_ADMIN()) {
		// Not admin or invalid IDs array
		return false;
	} elseif (!is_array($IDs)) {
		// No array
		return false;
	} elseif (count($IDs) == 0) {
		// Empty array
		return false;
	}

	// Set to true to make AND expression valid if first URL got unlocked
	$done = true;

	// Update the status for all ids
	foreach ($IDs as $id => $dummy) {
		// Test all ids through (ignores failed)
		$done = (($done) && (SURFBAR_CHANGE_STATUS($id, "PENDING", "REJECTED")));
	} // END - if

	// Return total status
	return $done;
}
//
// -----------------------------------------------------------------------------
//                               Member functions
// -----------------------------------------------------------------------------
//
// Member has added an URL
function SURFBAR_MEMBER_ADD_URL ($url, $limit) {
	global $_CONFIG;

	// Do some pre-checks
	if (!IS_MEMBER()) {
		// Not a member
		return false;
	} elseif (!VALIDATE_URL($url)) {
		// URL invalid
		return false;
	} elseif (SURFBAR_LOOKUP_BY_URL($url, $GLOBALS['userid'])) {
		// URL already found in surfbar!
		return false;
	} elseif (!SURFBAR_IF_USER_BOOK_MORE_URLS($GLOBALS['userid'])) {
		// No more allowed!
		return false;
	} elseif ("".($limit + 0)."" != "".$limit."") {
		// Invalid amount entered
		return false;
	}

	// Register the new URL
	return SURFBAR_REGISTER_URL($url, $GLOBALS['userid'], "PENDING", "reg", $limit);
}
// Create list of actions depending on status for the user
function SURFBAR_MEMBER_ACTIONS ($urlId, $status) {
	// Load all actions in an array for given status
	$actionArray = SURFBAR_GET_ACTION_ARRAY($status);

	// Init HTML code
	$OUT = "<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">
<tr>\n";

	// Calculate width
	$width = round(100 / count($actionArray));

	// "Walk" through all actions and create forms
	foreach ($actionArray as $actionId=>$action) {
		// Add form for this action
		$OUT .= sprintf("	<td align=\"center\" width=\"%d%%\">
		<form action=\"".URL."/modules.php?module=login&amp;what=surfbar_list\" method=\"post\" style=\"padding-bottom:0px\">
			<input type=\"hidden\" name=\"id\" value=\"%s\" />
			<input type=\"hidden\" name=\"action\" value=\"%s\" />
			<input type=\"submit\" class=\"member_submit\" name=\"ok\" title=\"{--MEMBER_SURFBAR_ACTION_%s_TITLE--}\" value=\"{--MEMBER_SURFBAR_ACTION_%s_SUBMIT--}\" />
		</form>
	</td>\n",
			$width,
			bigintval($urlId),
			strtolower($action),
			strtoupper($action),
			strtoupper($action)
		);
	} // END - foreach

	// Close table
	$OUT .= "</tr>
</table>\n";

	// Return code
	return $OUT;
}
// Do the member form request
function SURFBAR_MEMBER_DO_FORM ($formData, $URLs) {
	global $SURFBAR_CACHE;

	// By default no action is performed
	$performed = false;

	// Is this a member?
	if (!IS_MEMBER()) {
		// No member!
		return false;
	} elseif ((!isset($formData['id'])) || (!isset($formData['action']))) {
		// Important form elements are missing!
		return false;
	} elseif (!isset($URLs[$formData['id']])) {
		// ID not found in cache
		return false;
	} elseif (!SURFBAR_VALIDATE_MEMBER_ACTION_STATUS($formData['action'], $URLs[$formData['id']]['status'])) {
		// Action not allowed for current URL status
		return false;
	}

	// Create the function name for selected action
	$functionName = sprintf("SURFBAR_MEMBER_%s_ACTION", strtoupper(SQL_ESCAPE($formData['action'])));

	// Is the function there?
	if (function_exists($functionName)) {
		// Add new status
		$URLs[$formData['id']]['new_status'] = $SURFBAR_CACHE['new_status'];

		// Extract URL data for call-back
		$urlData = array($URLs[$formData['id']]);

		// Action found so execute it
		$performed = call_user_func_array($functionName, $urlData);
	} else {
		// Log invalid request
		DEBUG_LOG(__FUNCTION__.": action={$formData['action']},id={$formData['id']},function={$functionName}");
	}

	// Return status
	return $performed;
}
// Validate if the requested action can be performed on current URL status
function SURFBAR_VALIDATE_MEMBER_ACTION_STATUS ($action, $status) {
	global $SURFBAR_CACHE;

	// Search for the requested action/status combination in database
	$result = SQL_QUERY_ESC("SELECT new_status FROM "._MYSQL_PREFIX."_surfbar_actions WHERE action='%s' AND status='%s' LIMIT 1",
		array($action, $status), __FILE__, __LINE__);

	// Is the entry there?
	$isValid = (SQL_NUMROWS($result) == 1);

	// Fetch the new status if found
	if ($isValid) {
		list($SURFBAR_CACHE['new_status']) = SQL_FETCHROW($result);
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return status
	return $isValid;
}
//
// -----------------------------------------------------------------------------
//                               Member actions
// -----------------------------------------------------------------------------
//
// Retreat an URL
function SURFBAR_MEMBER_RETREAT_ACTION ($urlData) {
	// Create the data array for next function call
	$data = array(
		$urlData['id'] => $urlData
	);

	// Simply change the status here
	return SURFBAR_CHANGE_STATUS ($urlData['id'], $urlData['status'], $urlData['new_status'], $data);
}
// Book an URL now (from migration)
function SURFBAR_MEMBER_BOOKNOW_ACTION ($urlData) {
	// Create the data array for next function call
	$data = array(
		$urlData['id'] => $urlData
	);

	// Simply change the status here
	return SURFBAR_CHANGE_STATUS ($urlData['id'], $urlData['status'], $urlData['new_status'], $data);
}
//
// -----------------------------------------------------------------------------
//                           Self-maintenance functions
// -----------------------------------------------------------------------------
//
// Main function
function SURFBAR_HANDLE_SELF_MAINTENANCE () {
	// Handle URLs which limit has depleted so we can stop them
	SURFBAR_HANDLE_DEPLETED_VIEWS();

	// Handle low-points amounts
	SURFBAR_HANDLE_LOW_POINTS();
}
// Handle URLs which limit has depleted
function SURFBAR_HANDLE_DEPLETED_VIEWS () {
	// Get all URLs
	$urlArray = SURFBAR_GET_URL_DATA("0", "views_max", "id", "ASC", "id", " AND views_allowed>0 AND status='ACTIVE'");

	// Do we have some entries?
	if (count($urlArray) > 0) {
		// Then handle all!
		foreach ($urlArray as $id => $urlData) {
			// Backup data
			$data = $urlData;

			// Rewrite array for next call
			$urlData[$id] = $data;

			// Handle the status
			SURFBAR_CHANGE_STATUS($id, "ACTIVE", "DEPLETED", $urlData);
		} // END - foreach
	} // END - if
}
// Alert users which have URLs booked and are low on points amount
function SURFBAR_HANDLE_LOW_POINTS () {
	global $_CONFIG;

	// Get all userids
	$UIDs = SURFBAR_DETERMINE_DEPLETED_USERIDS($_CONFIG['surfbar_warn_low_points']);

	// "Walk" through all URLs
	foreach ($UIDs['uid'] as $uid => $dummy) {
		// Is the last notification far enougth away to notify again?
		if ((time() - $UIDs['notified'][$uid]) >= $_CONFIG['surfbar_low_interval']) {
			// Prepare content
			$content = array(
				'uid'      => $uid,
				'low'      => TRANSLATE_COMMA($_CONFIG['surfbar_warn_low_points']),
				'points'   => TRANSLATE_COMMA($UIDs['points'][$uid]),
				'notified' => MAKE_DATETIME($UIDs['notified'][$uid]),
				'interval' => CREATE_FANCY_TIME($_CONFIG['surfbar_low_interval'])
			);

			// Notify this user
			SURFBAR_NOTIFY_USER("low_points", $content);

			// Update last notified
			SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_user_data SET surfbar_low_notified=NOW() WHERE userid=%s LIMIT 1",
				array($uid), __FILE__, __LINE__);
		} // END - if
	} // END - foreach
}
//
// -----------------------------------------------------------------------------
//                               Generic functions
// -----------------------------------------------------------------------------
//
// Looks up by an URL
function SURFBAR_LOOKUP_BY_URL ($url, $uid) {
	// Now lookup that given URL by itself
	$urlArray = SURFBAR_GET_URL_DATA($url, "url", "id", "ASC", "id", sprintf(" AND userid=%s", bigintval($uid)));

	// Was it found?
	return (count($urlArray) > 0);
}
// Load URL data by given search term and column
function SURFBAR_GET_URL_DATA ($searchTerm, $column="id", $order="id", $sort="ASC", $group="id", $add="") {
	global $lastUrlData;

	// By default nothing is found
	$lastUrlData = array();

	// Is the column an id number?
	if (($column == "id") || ($column == "userid")) {
		// Extra secure input
		$searchTerm = bigintval($searchTerm);
	} // END - if

	// If the column is "id" there can be only one entry
	$limit = "";
	if ($column == "id") {
		$limit = "LIMIT 1";
	} // END - if

	// Look up the record
	$result = SQL_QUERY_ESC("SELECT id, userid, url, views_total, status, registered, last_locked, lock_reason, views_max, views_allowed
FROM "._MYSQL_PREFIX."_surfbar_urls
WHERE %s='%s'".$add."
ORDER BY %s %s
%s",
		array($column, $searchTerm, $order, $sort, $limit), __FILE__, __LINE__);

	// Is there at least one record?
	if (SQL_NUMROWS($result) > 0) {
		// Then load all!
		while ($dataRow = SQL_FETCHARRAY($result)) {
			// Shall we group these results?
			if ($group == "id") {
				// Add the row by id as index
				$lastUrlData[$dataRow['id']] = $dataRow;
			} else {
				// Group entries
				$lastUrlData[$dataRow[$group]][$dataRow['id']] = $dataRow;
			}
		} // END - while
	} // END - if

	// Free the result
	SQL_FREERESULT($result);

	// Return the result
	return $lastUrlData;
}
// Registers an URL with the surfbar. You should have called SURFBAR_LOOKUP_BY_URL() first!
function SURFBAR_REGISTER_URL ($url, $uid, $status="PENDING", $addMode="reg", $limit=0) {
	global $_CONFIG;

	// Make sure by the user registered URLs are always pending
	if ($addMode == "reg") $status = "PENDING";

	// Prepare content
	$content = array(
		'url'         => $url,
		'frametester' => FRAMETESTER($url),
		'uid'         => $uid,
		'status'      => $status,
		'limit'       => $limit
	);

	// Insert the URL into database
	$content['insert_id'] = SURFBAR_INSERT_URL_BY_ARRAY($content);

	// Is this ID valid?
	if ($content['insert_id'] == 0) {
		// INSERT did not insert any data!
		return false;
	} // END - if

	// Translate status and limit
	$content['status'] = SURFBAR_TRANSLATE_STATUS($content['status']);
	$content['limit'] = SURFBAR_TRANSLATE_LIMIT($content['limit']);

	// If in reg-mode we notify admin
	if (($addMode == "reg") || ($_CONFIG['surfbar_notify_admin_unlock'] == "Y")) {
		// Notify admin even when he as unlocked an email
		SURFBAR_NOTIFY_ADMIN("url_{$addMode}", $content);
	} // END - if

	// Send mail to user
	SURFBAR_NOTIFY_USER("url_{$addMode}", $content);

	// Return the insert id
	return $content['insert_id'];
}
// Inserts an url by given data array and return the insert id
function SURFBAR_INSERT_URL_BY_ARRAY ($urlData) {
	// Get userid
	$uid = bigintval($urlData['uid']);

	// Is the id set?
	if (empty($uid)) $uid = 0;

	// Just run the insert query for now
	SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_surfbar_urls (userid,url,status,views_max,views_allowed) VALUES (%s,'%s','%s','%s','%s')",
		array(
			$uid,
			$urlData['url'],
			$urlData['status'],
			$urlData['limit'],
			$urlData['limit']
		), __FILE__, __LINE__
	);

	// Return insert id
	return SQL_INSERTID();
}
// Notify admin(s) with a selected message and content
function SURFBAR_NOTIFY_ADMIN ($messageType, $content) {
	// Prepare template name
	$templateName = sprintf("admin_surfbar_%s", $messageType);

	// Set default subject if following eval() wents wrong
	$subject = ADMIN_SURFBAR_NOTIFY_DEFAULT_SUBJECT;

	// Prepare subject
	$subject = constant(sprintf("ADMIN_SURFBAR_NOTIFY_%s_SUBJECT",
		strtoupper($messageType)
	));

	// Send the notification out
	return SEND_ADMIN_NOTIFICATION($subject, $templateName, $content, $content['uid']);
}
// Notify the user about the performed action
function SURFBAR_NOTIFY_USER ($messageType, $content) {
	// Skip notification if userid is zero
	if ($content['uid'] == 0) {
		return false;
	} // END - if

	// Prepare template name
	$templateName = sprintf("member_surfbar_%s", $messageType);

	// Set default subject if following eval() wents wrong
	$subject = MEMBER_SURFBAR_NOTIFY_DEFAULT_SUBJECT;

	// Prepare subject
	$subject = constant(sprintf("MEMBER_SURFBAR_NOTIFY_%s_SUBJECT",
		strtoupper($messageType)
	));

	// Load template
	$mailText = LOAD_EMAIL_TEMPLATE($templateName, $content, $content['uid']);

	// Send the email
	return SEND_EMAIL($content['uid'], $subject, $mailText);
}
// Translates the limit
function SURFBAR_TRANSLATE_LIMIT ($limit) {
	// Is this zero?
	if ($limit == 0) {
		// Unlimited!
		$return = MEMBER_SURFBAR_UNLIMITED_VIEWS;
	} else {
		// Translate comma
		$return = TRANSLATE_COMMA($limit);
	}

	// Return value
	return $return;
}
// Translate the URL status
function SURFBAR_TRANSLATE_STATUS ($status) {
	// Create constant name
	$constantName = sprintf("SURFBAR_URL_STATUS_%s", strtoupper($status));

	// Set default translated status
	$statusTranslated = "!".$constantName."!";

	// Generate eval() command
	if (defined($constantName)) {
		$eval = "\$statusTranslated = ".$constantName.";";
		eval($eval);
	} // END - if

	// Return result
	return $statusTranslated;
}
// Determine reward
function SURFBAR_DETERMINE_REWARD ($onlyMin=false) {
	global $_CONFIG;

	// Static values are default
	$reward = $_CONFIG['surfbar_static_reward'];

	// Do we have static or dynamic?
	if ($_CONFIG['surfbar_pay_model'] == "DYNAMIC") {
		// "Calculate" dynamic reward
		if ($onlyMin) {
			$reward += SURFBAR_CALCULATE_DYNAMIC_MIN_VALUE();
		} else {
			$reward += SURFBAR_CALCULATE_DYNAMIC_ADD();
		}
	} // END - if

	// Return reward
	return $reward;
}
// Determine costs
function SURFBAR_DETERMINE_COSTS ($onlyMin=false) {
	global $_CONFIG;

	// Static costs is default
	$costs  = $_CONFIG['surfbar_static_costs'];

	// Do we have static or dynamic?
	if ($_CONFIG['surfbar_pay_model'] == "DYNAMIC") {
		// "Calculate" dynamic costs
		if ($onlyMin) {
			$costs += SURFBAR_CALCULATE_DYNAMIC_MIN_VALUE();
		} else {
			$costs += SURFBAR_CALCULATE_DYNAMIC_ADD();
		}
	} // END - if

	// Return costs
	return $costs;
}
// "Calculate" dynamic add
function SURFBAR_CALCULATE_DYNAMIC_ADD () {
	// Get min/max values
	$min = SURFBAR_CALCULATE_DYNAMIC_MIN_VALUE();
	$max = SURFBAR_CALCULATE_DYNAMIC_MAX_VALUE();

	// "Calculate" dynamic part and return it
	return mt_rand($min, $max);
}
// Determine right template name
function SURFBAR_DETERMINE_TEMPLATE_NAME() {
	// Default is the frameset
	$templateName = "surfbar_frameset";

	// Any frame set? ;-)
	if (isset($_GET['frame'])) {
		// Use the frame as a template name part... ;-)
		$templateName = sprintf("surfbar_frame_%s",
			SQL_ESCAPE($_GET['frame'])
		);
	} // END - if

	// Return result
	return $templateName;
}
// Check if the "reload lock" of the current user is full, call this function
// before you call SURFBAR_CHECK_RELOAD_LOCK().
function SURFBAR_CHECK_RELOAD_FULL() {
	global $SURFBAR_CACHE, $_CONFIG;

	// Default is full!
	$isFull = true;

	// Cache static reload lock
	$SURFBAR_CACHE['surf_lock'] = $_CONFIG['surfbar_static_lock'];
	//DEBUG_LOG(__FUNCTION__.":Fixed surf lock is ".$_CONFIG['surfbar_static_lock']."");

	// Do we have dynamic model?
	if ($_CONFIG['surfbar_pay_model'] == "DYNAMIC") {
		// "Calculate" dynamic lock
		$SURFBAR_CACHE['surf_lock'] += SURFBAR_CALCULATE_DYNAMIC_ADD();
	} // END - if

	// Ask the database
	$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt FROM "._MYSQL_PREFIX."_surfbar_locks
WHERE userid=%s AND (UNIX_TIMESTAMP() - ".SURFBAR_GET_DATA('surf_lock').") < UNIX_TIMESTAMP(last_surfed)
LIMIT 1",
		array($GLOBALS['userid']), __FILE__, __LINE__
	);

	// Fetch row
	list($SURFBAR_CACHE['user_locks']) = SQL_FETCHROW($result);

	// Is it null?
	if (is_null($SURFBAR_CACHE['user_locks'])) {
		// Then fix it to zero!
		$SURFBAR_CACHE['user_locks'] = 0;
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Get total URLs
	$total = SURFBAR_GET_TOTAL_URLS();

	// Do we have some URLs in lock? Admins can always surf on own URLs!
	//DEBUG_LOG(__FUNCTION__.":userLocks=".SURFBAR_GET_DATA('user_locks').",total={$total}");
	$isFull = ((SURFBAR_GET_DATA('user_locks') == $total) && ($total > 0));

	// Return result
	return $isFull;
}
// Get total amount of URLs of given status for current user or of ACTIVE URLs by default
function SURFBAR_GET_TOTAL_URLS ($status="ACTIVE", $excludeUserId=0) {
	// Determine depleted user account
	$UIDs = SURFBAR_DETERMINE_DEPLETED_USERIDS();

	// Is the exlude userid set?
	if ($excludeUserId > 0) {
		// Then add it
		$UIDs['uid'][$excludeUserId] = $excludeUserId;
	} // END - if

	// Get amount from database
	$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt
FROM "._MYSQL_PREFIX."_surfbar_urls
WHERE userid NOT IN (".implode(",", $UIDs['uid']).") AND status='%s'",
		array($status), __FILE__, __LINE__
	);

	// Fetch row
	list($cnt) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $cnt;
}
// Check wether the user is allowed to book more URLs
function SURFBAR_IF_USER_BOOK_MORE_URLS ($uid=0) {
	global $_CONFIG;

	// Is this admin and userid is zero or does the user has some URLs left to book?
	return ((($uid == 0) && (IS_ADMIN())) || (SURFBAR_GET_TOTAL_USER_URLS($uid, "", array("REJECTED")) < $_CONFIG['surfbar_max_order']));
}
// Get total amount of URLs of given status for current user
function SURFBAR_GET_TOTAL_USER_URLS ($uid=0, $status="",$exclude="") {
	global $_CONFIG;

	// Is the user 0 and user is logged in?
	if (($uid == 0) && (IS_MEMBER())) {
		// Then use this userid
		$uid = $GLOBALS['userid'];
	} elseif ($uid == 0) {
		// Error!
		return ($_CONFIG['surfbar_max_order'] + 1);
	}

	// Default is all URLs
	$ADD = "";

	// Is the status set?
	if (is_array($status)) {
		// Only URLs with these status
		$ADD = sprintf(" AND status IN('%s')", implode("','", $status));
	} elseif (!empty($status)) {
		// Only URLs with this status
		$ADD = sprintf(" AND status='%s'", $status);
	} elseif (is_array($exclude)) {
		// Exclude URLs with these status
		$ADD = sprintf(" AND status NOT IN('%s')", implode("','", $exclude));
	} elseif (!empty($exclude)) {
		// Exclude URLs with this status
		$ADD = sprintf(" AND status != '%s'", $exclude);
	}

	// Get amount from database
	$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt
FROM "._MYSQL_PREFIX."_surfbar_urls
WHERE userid=%s".$ADD."
LIMIT %s",
		array($uid, $_CONFIG['surfbar_max_order']), __FILE__, __LINE__
	);

	// Fetch row
	list($cnt) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $cnt;
}
// Generate a validation code for the given id number
function SURFBAR_GENERATE_VALIDATION_CODE ($urlId, $salt="") {
	global $_CONFIG, $SURFBAR_CACHE;

	// @TODO Invalid salt should be refused
	$SURFBAR_CACHE['salt'] = "INVALID";

	// Get code length from config
	$length = $_CONFIG['code_length'];

	// Fix length to 10
	if ($length == 0) $length = 10;

	// Generate a code until the length matches
	$valCode = "";
	while (strlen($valCode) != $length) {
		// Is the salt set?
		if (empty($salt)) {
			// Generate random hashed string
			$SURFBAR_CACHE['salt'] = sha1(GEN_PASS(255));
			//DEBUG_LOG(__FUNCTION__.":newSalt=".SURFBAR_GET_SALT()."");
		} else {
			// Use this as salt!
			$SURFBAR_CACHE['salt'] = $salt;
			//DEBUG_LOG(__FUNCTION__.":oldSalt=".SURFBAR_GET_SALT()."");
		}

		// ... and now the validation code
		$valCode = GEN_RANDOM_CODE($length, sha1(SURFBAR_GET_SALT().":".$urlId), $GLOBALS['userid']);
		//DEBUG_LOG(__FUNCTION__.":valCode={$valCode}");
	} // END - while

	// Hash it with md5() and salt it with the random string
	$hashedCode = generateHash(md5($valCode), SURFBAR_GET_SALT());

	// Finally encrypt it PGP-like and return it
	$valHashedCode = generatePassString($hashedCode);

	// Return hashed value
	//DEBUG_LOG(__FUNCTION__.":finalValCode={$valHashedCode}");
	return $valHashedCode;
}
// Check validation code
function SURFBAR_CHECK_VALIDATION_CODE ($urlId, $check, $salt) {
	global $SURFBAR_CACHE;

	// Secure id number
	$urlId = bigintval($urlId);

	// Now generate the code again
	$code = SURFBAR_GENERATE_VALIDATION_CODE($urlId, $salt);

	// Return result of checking hashes and salts
	//DEBUG_LOG(__FUNCTION__.":---".$code."|".$check."---");
	//DEBUG_LOG(__FUNCTION__.":+++".$salt."|".SURFBAR_GET_DATA('last_salt')."+++");
	return (($code == $check) && ($salt == SURFBAR_GET_DATA('last_salt')));
}
// Lockdown the userid/id combination (reload lock)
function SURFBAR_LOCKDOWN_ID ($urlId) {
	//* //DEBUG: */ print "LOCK!");
	///* //DEBUG: */ return;
	// Just add it to the database
	SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_surfbar_locks (userid, url_id) VALUES (%s, %s)",
		array($GLOBALS['userid'], bigintval($urlId)), __FILE__, __LINE__);

	// Remove the salt from database
	SQL_QUERY_ESC("DELETE LOW_PRIORITY FROM "._MYSQL_PREFIX."_surfbar_salts WHERE url_id=%s AND userid=%s LIMIT 1",
		array(bigintval($urlId), $GLOBALS['userid']), __FILE__, __LINE__);
}
// Pay points to the user and remove it from the sender
function SURFBAR_PAY_POINTS ($urlId) {
	// Remove it from the URL owner
	//DEBUG_LOG(__FUNCTION__.":uid=".SURFBAR_GET_USERID().",costs=".SURFBAR_GET_COSTS()."");
	if (SURFBAR_GET_USERID() > 0) {
		SUB_POINTS(SURFBAR_GET_USERID(), SURFBAR_GET_COSTS());
	} // END - if

	// Book it to the user
	//DEBUG_LOG(__FUNCTION__.":uid=".$GLOBALS['userid'].",reward=".SURFBAR_GET_REWARD()."");
	ADD_POINTS_REFSYSTEM($GLOBALS['userid'], SURFBAR_GET_DATA('reward'));
}
// Updates the statistics of current URL/userid
function SURFBAR_UPDATE_INSERT_STATS_RECORD () {
	global $_CONFIG;

	// Init add
	$ADD = "";

	// Get allowed views
	$allowed = SURFBAR_GET_ALLOWED_VIEWS();

	// Do we have a limit?
	if ($allowed > 0) {
		// Then count views_max down!
		$ADD .= ",views_max=views_max-1";
	} // END - if

	// Update URL stats
	SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_surfbar_urls SET views_total=views_total+1".$ADD." WHERE id=%s LIMIT 1",
		array(SURFBAR_GET_ID()), __FILE__, __LINE__);

	// Update the stats entry
	SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_surfbar_stats SET count=count+1 WHERE userid=%s AND url_id=%s LIMIT 1",
		array($GLOBALS['userid'], SURFBAR_GET_ID()), __FILE__, __LINE__);

	// Was that update okay?
	if (SQL_AFFECTEDROWS() < 1) {
		// No, then insert entry
		SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_surfbar_stats (userid,url_id,count) VALUES (%s,%s,1)",
			array($GLOBALS['userid'], SURFBAR_GET_ID()), __FILE__, __LINE__);
	} // END - if

	// Update total/daily/weekly/monthly counter
	$_CONFIG['surfbar_total_counter']++;
	$_CONFIG['surfbar_daily_counter']++;
	$_CONFIG['surfbar_weekly_counter']++;
	$_CONFIG['surfbar_monthly_counter']++;

	// Update config as well
	UPDATE_CONFIG(array("surfbar_total_counter", "surfbar_daily_counter", "surfbar_weekly_counter", "surfbar_monthly_counter"), array(1,1,1,1), "+");
}
// Update the salt for validation and statistics
function SURFBAR_UPDATE_SALT_STATS () {
	// Update statistics record
	SURFBAR_UPDATE_INSERT_STATS_RECORD();

	// Simply store the salt from cache away in database...
	SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_surfbar_salts SET last_salt='%s' WHERE url_id=%s AND userid=%s LIMIT 1",
		array(SURFBAR_GET_SALT(), SURFBAR_GET_ID(), $GLOBALS['userid']), __FILE__, __LINE__);

	// Debug message
	//DEBUG_LOG(__FUNCTION__.":salt=".SURFBAR_GET_SALT().",id=".SURFBAR_GET_ID().",uid=".$GLOBALS['userid']."");

	// Was that okay?
	if (SQL_AFFECTEDROWS() < 1) {
		// Insert missing entry!
		SQL_QUERY_ESC("INSERT INTO "._MYSQL_PREFIX."_surfbar_salts (url_id,userid,last_salt) VALUES (%s, %s, '%s')",
			array(SURFBAR_GET_ID(), $GLOBALS['userid'], SURFBAR_GET_SALT()), __FILE__, __LINE__);
	} // END - if

	// Debug message
	//DEBUG_LOG(__FUNCTION__.":affectedRows=".SQL_AFFECTEDROWS()."");

	// Return if the update was okay
	return (SQL_AFFECTEDROWS() == 1);
}
// Check if the reload lock is active for given id
function SURFBAR_CHECK_RELOAD_LOCK ($urlId) {
	//DEBUG_LOG(__FUNCTION__.":id={$urlId}");
	// Ask the database
	$result = SQL_QUERY_ESC("SELECT COUNT(id) AS cnt
FROM "._MYSQL_PREFIX."_surfbar_locks
WHERE userid=%s AND url_id=%s AND (UNIX_TIMESTAMP() - ".SURFBAR_GET_DATA('surf_lock').") < UNIX_TIMESTAMP(last_surfed)
ORDER BY last_surfed ASC
LIMIT 1",
		array($GLOBALS['userid'], bigintval($urlId)), __FILE__, __LINE__
	);

	// Fetch counter
	list($cnt) = SQL_FETCHROW($result);

	// Free result
	SQL_FREERESULT($result);

	// Return check
	//DEBUG_LOG(__FUNCTION__.":cnt={$cnt},".SURFBAR_GET_DATA('surf_lock')."");
	return ($cnt == 1);
}
// Determine which user hash no more points left
function SURFBAR_DETERMINE_DEPLETED_USERIDS ($limit=0) {
	// Init array
	$UIDs = array(
		'uid'      => array(),
		'points'   => array(),
		'notified' => array(),
	);

	// Do we have a current user id?
	if ((IS_MEMBER()) && ($limit == 0)) {
		// Then add this as well
		$UIDs['uid'][$GLOBALS['userid']]      = $GLOBALS['userid'];
		$UIDs['points'][$GLOBALS['userid']]   = GET_TOTAL_DATA($GLOBALS['userid'], "user_points", "points") - GET_TOTAL_DATA($GLOBALS['userid'], "user_data", "used_points");
		$UIDs['notified'][$GLOBALS['userid']] = 0;

		// Get all userid except logged in one
		$result = SQL_QUERY_ESC("SELECT u.userid, UNIX_TIMESTAMP(d.surfbar_low_notified) AS notified
FROM "._MYSQL_PREFIX."_surfbar_urls AS u
INNER JOIN "._MYSQL_PREFIX."_user_data AS d
ON u.userid=d.userid
WHERE u.userid NOT IN (%s,0) AND u.status='ACTIVE'
GROUP BY u.userid
ORDER BY u.userid ASC",
			array($GLOBALS['userid']), __FILE__, __LINE__);
	} else {
		// Get all userid
		$result = SQL_QUERY("SELECT u.userid, UNIX_TIMESTAMP(d.surfbar_low_notified) AS notified
FROM "._MYSQL_PREFIX."_surfbar_urls AS u
INNER JOIN "._MYSQL_PREFIX."_user_data AS d
ON u.userid=d.userid
WHERE u.status='ACTIVE'
GROUP BY u.userid
ORDER BY u.userid ASC", __FILE__, __LINE__);
	}

	// Load all userid
	while (list($uid, $notified) = SQL_FETCHROW($result)) {
		// Get total points
		$points = GET_TOTAL_DATA($uid, "user_points", "points") - GET_TOTAL_DATA($uid, "user_data", "used_points");
		//DEBUG_LOG(__FUNCTION__.":uid={$uid},points={$points}");

		// Shall we add this to ignore?
		if ($points <= $limit) {
			// Ignore this one!
			//DEBUG_LOG(__FUNCTION__.":uid={$uid} has depleted points amount!");
			$UIDs['uid'][$uid] = $uid;
			$UIDs['points'][$uid] = $points;
			$UIDs['notified'][$uid] = $notified;
		} // END - if
	} // END - while

	// Free result
	SQL_FREERESULT($result);

	// Debug message
	//DEBUG_LOG(__FUNCTION__.":UIDs::count=".count($UIDs)." (with own userid=".$GLOBALS['userid'].")");

	// Return result
	return $UIDs;
}
// Determine how many users are Online in surfbar
function SURFBAR_DETERMINE_TOTAL_ONLINE () {
	global $_CONFIG;

	// Count all users in surfbar modue and return the value
	$result = SQL_QUERY_ESC("SELECT id
FROM "._MYSQL_PREFIX."_surfbar_stats
WHERE (UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_online)) <= %s
GROUP BY userid",
		array($_CONFIG['online_timeout']), __FILE__, __LINE__);

	// Fetch count
	$cnt = SQL_NUMROWS($result);

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $cnt;
}
// Determine waiting time for one URL 
function SURFBAR_DETERMINE_WAIT_TIME () {
	global $_CONFIG;

	// Static time is default
	$time = $_CONFIG['surfbar_static_time'];

	// Which payment model do we have?
	if ($_CONFIG['surfbar_pay_model'] == "DYNAMIC") {
		// "Calculate" dynamic time
		$time += SURFBAR_CALCULATE_DYNAMIC_ADD();
	} // END - if

	// Return value
	return $time;
}
// Changes the status of an URL from given to other
function SURFBAR_CHANGE_STATUS ($urlId, $prevStatus, $newStatus, $data=array()) {
	global $_CONFIG;

	// Get URL data for status comparison if missing
	if (count($data) == 0) {
		$data = SURFBAR_GET_URL_DATA($urlId);
	} // END - if

	// Is the status like prevStatus is saying?
	if ($data[$urlId]['status'] != $prevStatus) {
		// No, then abort here
		return false;
	} // END - if


	// Update the status now
	// ---------------------- Commented out for debugging member actions! -----------------------
	//SQL_QUERY_ESC("UPDATE "._MYSQL_PREFIX."_surfbar_urls SET status='%s' WHERE id=%s LIMIT 1",
	//	array($newStatus, bigintval($urlId)), __FILE__, __LINE__);
	// ---------------------- Commented out for debugging member actions! -----------------------

	// Was that fine?
	if (SQL_AFFECTEDROWS() != 1) {
		// No, something went wrong
		return false;
	} // END - if

	// Prepare content for notification routines
	$data[$urlId]['uid']         = $data[$urlId]['userid'];
	$data[$urlId]['frametester'] = FRAMETESTER($data[$urlId]['url']);
	$data[$urlId]['reward']      = TRANSLATE_COMMA($_CONFIG['surfbar_static_reward']);
	$data[$urlId]['costs']       = TRANSLATE_COMMA($_CONFIG['surfbar_static_costs']);
	$data[$urlId]['status']      = SURFBAR_TRANSLATE_STATUS($newStatus);
	$data[$urlId]['registered']  = MAKE_DATETIME($data[$urlId]['registered'], "2");
	$newStatus = strtolower($newStatus);

	// Send admin notification
	SURFBAR_NOTIFY_ADMIN("url_{$newStatus}", $data[$urlId]);

	// Send user notification
	SURFBAR_NOTIFY_USER("url_{$newStatus}", $data[$urlId]);

	// All done!
	return true;
}
// Calculate minimum value for dynamic payment model
function SURFBAR_CALCULATE_DYNAMIC_MIN_VALUE () {
	global $_CONFIG;

	// Addon is zero by default
	$addon = 0;

	// Percentage part
	$percent = abs(log($_CONFIG['surfbar_dynamic_percent'] / 100 + 1));

	// Get total users
	$totalUsers = GET_TOTAL_DATA("CONFIRMED", "user_data", "userid", "status", true);

	// Get online users
	$onlineUsers = SURFBAR_DETERMINE_TOTAL_ONLINE();

	// Calculate addon
	$addon += abs(log($onlineUsers / $totalUsers + 1) * $percent * $totalUsers);

	// Get total URLs
	$totalUrls = SURFBAR_GET_TOTAL_URLS("ACTIVE", "0");

	// Get user's total URLs
	$userUrls = SURFBAR_GET_TOTAL_USER_URLS(0, "ACTIVE");

	// Calculate addon
	if ($totalUrls > 0) {
		$addon += abs(log($userUrls / $totalUrls + 1) * $percent * $totalUrls);
	} else {
		$addon += abs(log($userUrls / 1 + 1) * $percent * $totalUrls);
	}

	// Return addon
	return $addon;
}
// Calculate maximum value for dynamic payment model
function SURFBAR_CALCULATE_DYNAMIC_MAX_VALUE () {
	global $_CONFIG;

	// Addon is zero by default
	$addon = 0;

	// Maximum value
	$max = log(2);

	// Percentage part
	$percent = abs(log($_CONFIG['surfbar_dynamic_percent'] / 100 + 1));

	// Get total users
	$totalUsers = GET_TOTAL_DATA("CONFIRMED", "user_data", "userid", "status", true);

	// Calculate addon
	$addon += abs($max * $percent * $totalUsers);

	// Get total URLs
	$totalUrls = SURFBAR_GET_TOTAL_URLS("ACTIVE", "0");

	// Calculate addon
	$addon += abs($max * $percent * $totalUrls);

	// Return addon
	return $addon;
}
// Calculate dynamic lock
function SURFBAR_CALCULATE_DYNAMIC_LOCK () {
	global $_CONFIG;

	// Default lock is 30 seconds
	$addon = 30;

	// Get online users
	$onlineUsers = SURFBAR_DETERMINE_TOTAL_ONLINE();

	// Calculate lock
	$addon = abs(log($onlineUsers / $addon + 1));

	// Return value
	return $addon;
}
// "Getter" for lock ids array
function SURFBAR_GET_LOCK_IDS () {
	// Prepare some arrays
	$IDs = array();
	$USE = array();
	$ignored = array();

	// Get all id from locks within the timestamp
	$result = SQL_QUERY_ESC("SELECT id, url_id, UNIX_TIMESTAMP(last_surfed) AS last
FROM
	"._MYSQL_PREFIX."_surfbar_locks
WHERE
	userid=%s
ORDER BY
	id ASC", array($GLOBALS['userid']),
		__FILE__, __LINE__);

	// Load all entries
	while (list($lid, $url, $last) = SQL_FETCHROW($result)) {
		// Debug message
		//DEBUG_LOG(__FUNCTION__.":next - lid={$lid},url={$url},rest=".(time() - $last)."/".SURFBAR_GET_DATA('surf_lock')."");

		// Skip entries that are too old
		if (($last > (time() - SURFBAR_GET_DATA('surf_lock'))) && (!in_array($url, $ignored))) {
			// Debug message
			//DEBUG_LOG(__FUNCTION__.":okay - lid={$lid},url={$url},last={$last}");

			// Add only if missing or bigger
			if ((!isset($IDs[$url])) || ($IDs[$url] > $last)) {
				// Debug message
				//DEBUG_LOG(__FUNCTION__.":ADD - lid={$lid},url={$url},last={$last}");

				// Add this ID
				$IDs[$url] = $last;
				$USE[$url] = $lid;
			} // END - if
		} else {
			// Debug message
			//DEBUG_LOG(__FUNCTION__.":ignore - lid={$lid},url={$url},last={$last}");

			// Ignore these old entries!
			$ignored[] = $url;
			unset($IDs[$url]);
			unset($USE[$url]);
		}
	} // END - while

	// Free result
	SQL_FREERESULT($result);

	// Return array
	return $USE;
}
// "Getter" for maximum random number
function SURFBAR_GET_MAX_RANDOM ($UIDs, $ADD) {
	global $_CONFIG;
	// Count max availabe entries
	$result = SQL_QUERY("SELECT sbu.id AS cnt
FROM "._MYSQL_PREFIX."_surfbar_urls AS sbu
LEFT JOIN "._MYSQL_PREFIX."_surfbar_salts AS sbs
ON sbu.id=sbs.url_id
LEFT JOIN "._MYSQL_PREFIX."_surfbar_locks AS l
ON sbu.id=l.url_id
WHERE sbu.userid NOT IN (".implode(",", $UIDs).") AND (sbu.views_allowed=0 OR (sbu.views_allowed > 0 AND sbu.views_max > 0)) AND sbu.status='ACTIVE'".$ADD."
GROUP BY sbu.id", __FILE__, __LINE__);

	// Log last query
	//DEBUG_LOG(__FUNCTION__.":lastQuery=".$_CONFIG['db_last_query']."|numRows=".SQL_NUMROWS($result)."|Affected=".SQL_AFFECTEDROWS()."");

	// Fetch max rand
	$maxRand = SQL_NUMROWS($result);

	// Free result
	SQL_FREERESULT($result);

	// Return value
	return $maxRand;
}
// Load all URLs of the current user and return it as an array
function SURFBAR_GET_USER_URLS () {
	// Init array
	$URLs = array();

	// Begin the query
	$result = SQL_QUERY_ESC("SELECT u.id, u.userid, u.url, u.views_total, u.status, UNIX_TIMESTAMP(u.registered) AS registered, UNIX_TIMESTAMP(u.last_locked) AS last_locked, u.lock_reason AS lock_reason
FROM "._MYSQL_PREFIX."_surfbar_urls AS u
WHERE u.userid=%s AND u.status != 'DELETED'
ORDER BY u.id ASC",
		array($GLOBALS['userid']), __FILE__, __LINE__);

	// Are there entries?
	if (SQL_NUMROWS($result) > 0) {
		// Load all rows
		while ($row = SQL_FETCHARRAY($result)) {
			// Add the row
			$URLs[$row['id']] = $row;
		} // END - while
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return the array
	return $URLs;
}
// "Getter" for member action array for given status
function SURFBAR_GET_ACTION_ARRAY ($status) {
	// Init array
	$returnArray = array();

	// Get all assigned actions
	$result = SQL_QUERY_ESC("SELECT action FROM "._MYSQL_PREFIX."_surfbar_actions WHERE status='%s' ORDER BY id ASC",
		array($status), __FILE__, __LINE__);

	// Some entries there?
	if (SQL_NUMROWS($result) > 0) {
		// Load all actions
		while (list($action) = SQL_FETCHROW($result)) {
			$returnArray[] = $action;
		} // END - if
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	return $returnArray;
}
// Reload to configured stop page
function SURFBAR_RELOAD_TO_STOP_PAGE($page="stop") {
	global $_CONFIG;

	// Internal or external?
	if (($_CONFIG['surfbar_pause_mode'] == "INTERNAL") || (empty($_CONFIG['surfbar_pause_url']))) {
		// Reload to internal page
		LOAD_URL("surfbar.php?frame=".$page);
	} else {
		// Reload to external page
		LOAD_URL($_CONFIG['surfbar_pause_url']);
	}
}

// Determine next id for surfbar or get data for given id, always call this before you call other
// getters below this function!!!
function SURFBAR_DETERMINE_NEXT_ID ($urlId = 0) {
	global $SURFBAR_CACHE, $_CONFIG;

	// Default is no id and no random number
	$nextId = 0;
	$randNum = 0;

	// Is the ID set?
	if ($urlId == 0) {
		// Get array with lock ids
		$USE = SURFBAR_GET_LOCK_IDS();

		// Shall we add some URL ids to ignore?
		$ADD = "";
		if (count($USE) > 0) {
			// Ignore some!
			$ADD = " AND sbu.id NOT IN (";
			foreach ($USE as $url_id => $lid) {
				// Add URL id
				$ADD .= $url_id.",";
			} // END - foreach

			// Add closing bracket
			$ADD = substr($ADD, 0, -1) . ")";
		} // END - if

		// Determine depleted user account
		$UIDs = SURFBAR_DETERMINE_DEPLETED_USERIDS();

		// Get maximum randomness factor
		$maxRand = SURFBAR_GET_MAX_RANDOM($UIDs['uid'], $ADD);

		// If more than one URL can be called generate the random number!
		if ($maxRand > 1) {
			// Generate random number
			$randNum = mt_rand(0, ($maxRand - 1));
		} // END - if

		// And query the database
		//DEBUG_LOG(__FUNCTION__.":randNum={$randNum},maxRand={$maxRand},surfLock=".SURFBAR_GET_DATA('surf_lock')."");
		$result = SQL_QUERY_ESC("SELECT sbu.id, sbu.userid, sbu.url, sbs.last_salt, sbu.views_total, sbu.views_max, sbu.views_allowed, UNIX_TIMESTAMP(l.last_surfed) AS last_surfed
FROM "._MYSQL_PREFIX."_surfbar_urls AS sbu
LEFT JOIN "._MYSQL_PREFIX."_surfbar_salts AS sbs
ON sbu.id=sbs.url_id
LEFT JOIN "._MYSQL_PREFIX."_surfbar_locks AS l
ON sbu.id=l.url_id
WHERE sbu.userid NOT IN (".implode(",", $UIDs['uid']).") AND sbu.status='ACTIVE' AND (sbu.views_allowed=0 OR (sbu.views_allowed > 0 AND sbu.views_max > 0))".$ADD."
GROUP BY sbu.id
ORDER BY l.last_surfed ASC, sbu.id ASC
LIMIT %s,1",
			array($randNum), __FILE__, __LINE__
		);
	} else {
		// Get data from specified id number
		$result = SQL_QUERY_ESC("SELECT sbu.id, sbu.userid, sbu.url, sbs.last_salt, sbu.views_total, sbu.views_max, sbu.views_allowed, UNIX_TIMESTAMP(l.last_surfed) AS last_surfed
FROM "._MYSQL_PREFIX."_surfbar_urls AS sbu
LEFT JOIN "._MYSQL_PREFIX."_surfbar_salts AS sbs
ON sbu.id=sbs.url_id
LEFT JOIN "._MYSQL_PREFIX."_surfbar_locks AS l
ON sbu.id=l.url_id
WHERE sbu.userid != %s AND sbu.status='ACTIVE' AND sbu.id=%s AND (sbu.views_allowed=0 OR (sbu.views_allowed > 0 AND sbu.views_max > 0))
LIMIT 1",
			array($GLOBALS['userid'], bigintval($urlId)), __FILE__, __LINE__
		);
	}

	// Is there an id number?
	//DEBUG_LOG(__FUNCTION__.":lastQuery=".$_CONFIG['db_last_query']."|numRows=".SQL_NUMROWS($result)."|Affected=".SQL_AFFECTEDROWS()."");
	if (SQL_NUMROWS($result) == 1) {
		// Load/cache data
		//DEBUG_LOG(__FUNCTION__.":count(".count($SURFBAR_CACHE).") - BEFORE");
		$SURFBAR_CACHE = merge_array($SURFBAR_CACHE, SQL_FETCHARRAY($result));
		//DEBUG_LOG(__FUNCTION__.":count(".count($SURFBAR_CACHE).") - AFTER");

		// Determine waiting time
		$SURFBAR_CACHE['time'] = SURFBAR_DETERMINE_WAIT_TIME();

		// Is the last salt there?
		if (is_null($SURFBAR_CACHE['last_salt'])) {
			// Then repair it wit the static!
			//DEBUG_LOG(__FUNCTION__.":last_salt - FIXED!");
			$SURFBAR_CACHE['last_salt'] = "";
		} // END - if

		// Fix missing last_surfed
		if ((!isset($SURFBAR_CACHE['last_surfed'])) || (is_null($SURFBAR_CACHE['last_surfed']))) {
			// Fix it here
			//DEBUG_LOG(__FUNCTION__.":last_surfed - FIXED!");
			$SURFBAR_CACHE['last_surfed'] = 0;
		} // END - if

		// Get base/fixed reward and costs
		$SURFBAR_CACHE['reward'] = SURFBAR_DETERMINE_REWARD();
		$SURFBAR_CACHE['costs']  = SURFBAR_DETERMINE_COSTS();
		//DEBUG_LOG(__FUNCTION__.":BASE/STATIC - reward=".SURFBAR_GET_REWARD()."|costs=".SURFBAR_GET_COSTS()."");

		// Only in dynamic model add the dynamic bonus!
		if ($_CONFIG['surfbar_pay_model'] == "DYNAMIC") {
			// Calculate dynamic reward/costs and add it
			$SURFBAR_CACHE['reward'] += SURFBAR_CALCULATE_DYNAMIC_ADD();
			$SURFBAR_CACHE['costs']  += SURFBAR_CALCULATE_DYNAMIC_ADD();
			//DEBUG_LOG(__FUNCTION__.":DYNAMIC+ - reward=".SURFBAR_GET_REWARD()."|costs=".SURFBAR_GET_COSTS()."");
		} // END - if

		// Now get the id
		$nextId = SURFBAR_GET_ID();
	} // END - if

	// Free result
	SQL_FREERESULT($result);

	// Return result
	//DEBUG_LOG(__FUNCTION__.":nextId={$nextId}");
	return $nextId;
}
// -----------------------------------------------------------------------------
// PLEASE DO NOT ADD ANY OTHER FUNCTIONS BELOW THIS LINE ELSE THEY "WRAP" THE
// $SURFBAR_CACHE ARRAY!
// -----------------------------------------------------------------------------
// Private getter for data elements
function SURFBAR_GET_DATA ($element) {
	global $SURFBAR_CACHE;
	//DEBUG_LOG(__FUNCTION__.":element={$element}");

	// Default is null
	$data = null;

	// Is the entry there?
	if (isset($SURFBAR_CACHE[$element])) {
		// Then take it
		$data = $SURFBAR_CACHE[$element];
	} else { // END - if
		print("<pre>");
		print_r($SURFBAR_CACHE);
		debug_print_backtrace();
		die("</pre>");
	}

	// Return result
	//DEBUG_LOG(__FUNCTION__.":element[$element]={$data}");
	return $data;
}
// Getter for reward from cache
function SURFBAR_GET_REWARD () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('reward');
}
// Getter for costs from cache
function SURFBAR_GET_COSTS () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('costs');
}
// Getter for URL from cache
function SURFBAR_GET_URL () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('url');
}
// Getter for salt from cache
function SURFBAR_GET_SALT () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('salt');
}
// Getter for id from cache
function SURFBAR_GET_ID () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('id');
}
// Getter for userid from cache
function SURFBAR_GET_USERID () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('userid');
}
// Getter for user reload locks
function SURFBAR_GET_USER_RELOAD_LOCK () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('user_locks');
}
// Getter for reload time
function SURFBAR_GET_RELOAD_TIME () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('time');
}
// Getter for allowed views
function SURFBAR_GET_ALLOWED_VIEWS () {
	// Get data element and return its contents
	return SURFBAR_GET_DATA('views_allowed');
}
//
?>
